// board.cpp
//  Boaard clss

#include "tinparty.h"

using namespace std;

vector<Board*> Board::boards;

uint32_t Board::RegisterBoard(Board* c)
{
	boards.push_back(c);
	c->id = boards.size() - 1;
	return c->id;
}

void Board::StaticInitGraphics()
{
	for(vector<Board*>::iterator i = boards.begin(); i != boards.end(); i++)
	{
		if(!(*i)->InitGraphics())
			i = boards.erase(i);
	}
}

void Board::StaticDestroy()
{
	for(vector<Board*>::iterator i = boards.begin(); i != boards.end(); i++)
		delete *i;
}

void Board::RealToScreen(int cx, int cy, int w, int h, int inx, int iny, int& x, int& y)
{
	// Convert absolute to screen
	int tlx = cx - (w / 2);
	int tly = cy - (h / 2);
	x = inx - tlx;
	y = iny - tly;
}

Board::Board()
{
}

EStatus Board::Tick(double dtime)
{
	return S_Continue;
}

void Board::Draw(BITMAP* dest, int camx, int camy, int x, int y, int w, int h)
{
}

class BoardSineWave: public Board
{
	public: // Constants
		const double period;
		const double amplitude;
		const int piecesperperiod;
		
	public: // Public variables
		BITMAP* portrait;
		
	public: // Public functions
		BoardSineWave(): period(800), amplitude(150), piecesperperiod(8)
		{ }
		virtual ~BoardSineWave()
		{
			destroy_bitmap(portrait);
			portrait = NULL;
		}
		virtual bool InitGraphics()
		{
			portrait = create_bitmap(128, 128);
			if(!portrait)
				cout << "Failed to create a bitmap" << endl;
				
			clear_to_color(portrait, 0);
			
			for(int i = 0; i < portrait->w; i++)
			{
				putpixel(portrait, i, (portrait->h / 2) + (uint32_t) (sin(2.28 * 3 * i / portrait->w) * (portrait->h / 4)), makecol(255, 255, 255));
			}
			
			return portrait;
		}
		virtual void DrawPortrait(BITMAP* dest, int x, int y, int w, int h)
		{
			stretch_blit(portrait, dest, 0,  0, portrait->w, portrait->h, x, y, w, h);
		}
		virtual void Draw(BITMAP* dest, int camx, int camy, int x, int y, int w, int h)
		{
			// Convert absolute to screen
			int tlx = camx - (w / 2);
			int tly = camy - (h / 2);
			int brx = camx + (w / 2);
			int bry = camy + (h / 2);
			// Make a fresh board
			clear_to_color(dest, makecol(255, 255, 255));
			// Draw our sine wave
			for(double i = tlx; i < brx && i < 0; i += .5)
				putpixel(dest, x - tlx + i, y - tly + (uint32_t) (sin(6.28 / period * i) * amplitude), 0);
			// Draw the start circle
			if(brx >= 0)
				circle(dest, x - tlx + amplitude, y - tly, amplitude, 0);
			// Draw our circles
			// Start of the period
			int sper = tlx / -period;
			for(double i = (sper * -period) - period; i < brx && i < 0; i += /*(1.5 + sin(6.28 / period * i / 4)) **/ period / piecesperperiod)
				circle(dest, x - tlx + i, y - tly + (uint32_t) (sin(6.28 / period * i) * amplitude), 20, makecol(255, 0, 0));
			Board::Draw(dest, camx, camy, x, y, w, h);
		}
		virtual void GetLocationOfSquare(uint32_t snum, int& x, int& y)
		{
			if(snum == 0)
			{
				x = amplitude;
				y = 0;
				return;
			}
			int cnum = 0;
			double i = 0;
			for(; cnum < snum; i += /*(1.5 + sin(6.28 / period * i / 4)) **/ period / piecesperperiod, cnum++);
			x = -i;
			y = -sin(6.28 / period * i) * amplitude;
		}
};
ADD_BOARD(BoardSineWave);

// The end
