// character.cpp
//  Characters!!!

#include "tinparty.h"

using namespace std;

vector<Character*> Character::chars;

uint32_t Character::RegisterCharacter(Character* c)
{
	chars.push_back(c);
	c->id = chars.size() - 1;
	return c->id;
}

void Character::StaticInitGraphics()
{
	for(vector<Character*>::iterator i = chars.begin(); i != chars.end(); i++)
	{
		if(!(*i)->InitGraphics())
			i = chars.erase(i);
	}
}

void Character::StaticDestroy()
{
	for(vector<Character*>::iterator i = chars.begin(); i != chars.end(); i++)
		delete *i;
}

Character* Character::GetByName(const char* name)
{
	for(vector<Character*>::iterator i = chars.begin(); i != chars.end(); i++)
	{
		if(stricmp((*i)->name, name) == 0)
			return *i;
	}
	return NULL;
}

Character::Character(): playable(true)
{
}

void Character::StandardStates(BITMAP* dest, int x, int y, int w, int h, BITMAP* frame)
{
	switch(state)
	{
		case S_Standing:
			stretch_sprite(dest, frame, x, y, w, h);
			break;
		case S_Walking:
			stretch_sprite(dest, frame, x, y - (uint32_t) abs(sin(6.14 * animphase) * .5 * h), w, h);
			break;
		/*{
			double scale;
			if(w > h) scale = w / frame->w;
			else scale = h / frame->h;
			rotate_scaled_sprite(dest, frame, x, y, ftofix(sin(phase * 10) * 256 / 18), ftofix(scale));
			stretch_sprite(dest, frame, x, y, w, h);
			cout << id << " ";
			break;
		}*/
		case S_Jumping:
			stretch_sprite(dest, frame, x, y - (uint32_t) (sin(3.14 * progress) * 1.5 * h), w, h);
			break;
	}
}

EStatus Character::Tick(double dtime)
{
	progress += dtime * timedialation;
	animphase += dtime * timedialation;
	if(progress >= 1 && state != S_Standing && state != S_Walking)
	{
		Animate(S_Standing, 1);
		return S_Next;
	}
	return S_Continue;
}

void Character::Face(EDirection newdir)
{
	dir = newdir;
}

void Character::Animate(EState animation, double timetoarrive)
{
	state = animation;
	progress = 0;
	animphase = 0;
	timedialation = 1 / timetoarrive;
}

class CharacterX: public Character
{
	public: // Public variables
		BITMAP* thex;
	public: // Public functions
		CharacterX()
		{ name = "TINS"; }
		virtual ~CharacterX()
		{ destroy_bitmap(thex); }
		virtual bool InitGraphics()
		{
			thex = create_bitmap(text_length(font, "TIN"), text_height(font));
			clear_to_color(thex, makecol(255, 255, 255));
			textout(thex, font, "TIN", 0, 0, makecol(255, 0, 0));
			return true;
		}
		virtual void DrawPortrait(BITMAP* dest, int x, int y, int w, int h)
		{
			stretch_blit(thex, dest, 0,  0, thex->w, thex->h, x, y, w, h);
		}
		virtual void DrawSelf(BITMAP* dest, int x, int y, int w, int h)
		{
			StandardStates(dest, x, y, w, h, thex);
		}
};
ADD_CHARACTER(CharacterX);

class CharacterPika: public Character
{
	public: // Public variables
		BITMAP* portrait;
		
	public: // Public functions
		CharacterPika()
		{ name = "Pikachu"; }
		virtual ~CharacterPika()
		{
			destroy_bitmap(portrait);
			portrait = NULL;
		}
		virtual bool InitGraphics()
		{
			portrait = load_bitmap("data/pikachu.tga", NULL);
			if(!portrait)
				cout << "Failed to load data/pikachu.tga" << endl;
			
			return portrait;
		}
		virtual void DrawPortrait(BITMAP* dest, int x, int y, int w, int h)
		{
			stretch_blit(portrait, dest, 0,  0, portrait->w, portrait->h, x, y, w, h);
		}
		virtual void DrawSelf(BITMAP* dest, int x, int y, int w, int h)
		{
			StandardStates(dest, x, y, w, h, portrait);
		}
};
ADD_CHARACTER(CharacterPika);

class CharacterJack: public Character
{
	public: // Public variables
		BITMAP* portrait;
		
	public: // Public functions
		CharacterJack()
		{ name = "Jack"; }
		virtual ~CharacterJack()
		{
			destroy_bitmap(portrait);
			portrait = NULL;
		}
		virtual bool InitGraphics()
		{
			portrait = load_bitmap("data/jack.tga", NULL);
			if(!portrait)
				cout << "Failed to load data/jack.tga" << endl;
			
			return portrait;
		}
		virtual void DrawPortrait(BITMAP* dest, int x, int y, int w, int h)
		{
			stretch_blit(portrait, dest, 0,  0, portrait->w, portrait->h, x, y, w, h);
		}
		virtual void DrawSelf(BITMAP* dest, int x, int y, int w, int h)
		{
			StandardStates(dest, x, y, w, h, portrait);
		}
};
ADD_CHARACTER(CharacterJack);

class CharacterNick: public Character
{
	public: // Public variables
		BITMAP* portrait;
		
	public: // Public functions
		CharacterNick()
		{ name = "Nick"; }
		virtual ~CharacterNick()
		{
			destroy_bitmap(portrait);
			portrait = NULL;
		}
		virtual bool InitGraphics()
		{
			portrait = load_bitmap("data/nick.tga", NULL);
			if(!portrait)
				cout << "Failed to load data/nick.tga" << endl;
			
			return portrait;
		}
		virtual void DrawPortrait(BITMAP* dest, int x, int y, int w, int h)
		{
			stretch_blit(portrait, dest, 0,  0, portrait->w, portrait->h, x, y, w, h);
		}
		virtual void DrawSelf(BITMAP* dest, int x, int y, int w, int h)
		{
			StandardStates(dest, x, y, w, h, portrait);
		}
};
ADD_CHARACTER(CharacterNick);

class CharacterTroll: public Character
{
	public: // Public variables
		BITMAP* portrait;
		
	public: // Public functions
		CharacterTroll(): Character(E_Unlockable)
		{ name = "Troll"; }
		virtual ~CharacterTroll()
		{
			destroy_bitmap(portrait);
			portrait = NULL;
		}
		virtual bool InitGraphics()
		{
			portrait = load_bitmap("data/troll.tga", NULL);
			if(!portrait)
				cout << "Failed to load data/troll.tga" << endl;
			
			return portrait;
		}
		virtual void DrawPortrait(BITMAP* dest, int x, int y, int w, int h)
		{
			stretch_blit(portrait, dest, 0,  0, portrait->w, portrait->h, x, y, w, h);
		}
		virtual void DrawSelf(BITMAP* dest, int x, int y, int w, int h)
		{
			StandardStates(dest, x, y, w, h, portrait);
		}
};
ADD_CHARACTER(CharacterTroll);

// The end
