// game.h
//  Game GameState

class Block
{
	public: // Public variables
		uint32_t min, max;
		uint32_t value;
		uint32_t pattachment;
		
		double progress;
		bool rolling;
		
	public: // Public functions
		Block();
		virtual ~Block()
		{ }
		
		virtual bool InitGraphics()
		{ return true; }
		
		virtual EStatus Tick(double dtime);
		virtual void Draw(BITMAP* dest, int x, int y, int w, int h);
		
		void Hit();
};

class HUD
{
	public: // Public variables
		Pawn* player;
		uint32_t num;
		
	public: // Public functions
		HUD();
		virtual ~HUD()
		{ }
		
		virtual bool InitGraphics()
		{ return true; }
		
		virtual EStatus Tick(double dtime);
		virtual void Draw(BITMAP* dest, int x, int y, int w, int h);
};

class TurnOrderFinder: public GameState
{
	public: // Public static functions
		static GameState* Factory()
		{ return new TurnOrderFinder; }
		
	public: // Public variables
 		std::stringstream msgtext;
		SubState* textbox;
		std::vector<Block*> blocks;
		uint32_t ndone, ndone2, ndone3;
		bool counting;
		int camx, camy;
		double progress;
		
	public: // Public functions
		TurnOrderFinder(): textbox(NULL), ndone(0), ndone2(0), ndone3(0), counting(false), camx(0), camy(0), progress(0)
		{ }
		virtual ~TurnOrderFinder();
		
		virtual bool InitLogic();
		virtual bool InitGraphics();
		
		virtual EStatus Tick(double dtime);
		virtual void Draw(BITMAP* dest);
};

class MainGame: public GameState
{
	public: // Public static functions
		static GameState* Factory()
		{ return new MainGame; }
		
	public: // Public variables
		Block block;
		uint32_t curturn;
		int movesleft;
		double progress;
		int camx, camy;
		bool paused;
		enum EState
		{
			PreRoll,
			Rolling,
			Wait,
			TrackingPlayer,
			Wait2,
		} state;
		
	public: // Public functions
		MainGame(): curturn(0), state(PreRoll), paused(false)
		{ }
		virtual ~MainGame()
		{ }
		
		virtual bool InitLogic();
		virtual bool InitGraphics();
		
		void EnterState(EState news);
		
		virtual EStatus Tick(double dtime);
		virtual void Draw(BITMAP* dest);
};

class EndGame: public GameState
{
	public: // Public static functions
		static GameState* Factory()
		{ return new EndGame; }
		
	public: // Public variables
		SubState* textbox;
		double progress;
		bool counting;
		int p1, p2, p3, p4;
		enum EState
		{
			Suspense,
			Announce4,
			Announce3,
			Announce2,
			Announce1,
			GoodGame,
		} state;
		
	public: // Public functions
		EndGame(): textbox(NULL), counting(false), state(Suspense)
		{ }
		virtual ~EndGame()
		{ }
		
		virtual bool InitLogic();
		virtual bool InitGraphics();
		
		void EnterState(EState news);
		
		virtual EStatus Tick(double dtime);
		virtual void Draw(BITMAP* dest);
};

// The end
