// menu.h
//  Menu GameState

#include "tinparty.h"

using namespace std;

Menu::~Menu()
{
	destroy_bitmap(gradient);
	gradient = NULL;
	destroy_bitmap(fade_gradient);
	fade_gradient = NULL;
	destroy_bitmap(splash);
	splash = NULL;
}

bool Menu::InitGraphics()
{
	gradient = create_bitmap(SCREEN_W, 1);
	if(!gradient)
		cout << "Failed to make a bitmap" << endl;
	for(uint32_t i = 0; i < SCREEN_W; i++)
	{
		int r, g, b;
		hsv_to_rgb((double) i / SCREEN_W * 300, 1, 1, &r, &g, &b);
		putpixel(gradient, i, 0, makecol(r, g, b));
	}
	fade_gradient = create_bitmap(SCREEN_W, 1);
	if(!fade_gradient)
		cout << "Failed to make a fade bitmap" << endl;
	for(uint32_t i = 0; i < SCREEN_W; i++)
	{
		int r, g, b;
		hsv_to_rgb((double) i / SCREEN_W * 300, .25, 1, &r, &g, &b);
		putpixel(fade_gradient, i, 0, makecol(r, g, b));
	}
	splash = load_bitmap("data/splash.tga", NULL);
	if(!splash)
		cout << "Failed to load data/splash.tga" << endl;
	
	return gradient && fade_gradient && splash;
}

EStatus Menu::Tick(double dtime)
{
	switch(curscreen)
	{
		case TitleScreen:
			if(keypressed())
			{
				if(readkey() >> 8 == KEY_ESC) return S_Quit;
				curscreen = MainScreen;
			}
			return S_Continue;
			
		case MainScreen:
			if(keypressed())
			{
				int key = readkey();
				if(key >> 8 == KEY_ESC)
				{
					//SOUND menu back
					return S_Quit;
				}
				if(key >> 8 == KEY_DOWN)
				{
					caret_position++;
					if(caret_position > 3) caret_position = 0;
					full_redraw = true;
					//SOUND menu move
				}
				if(key >> 8 == KEY_UP)
				{
					if(caret_position <= 0) caret_position = 4;
					caret_position--;
					full_redraw = true;
					//SOUND menu move
				}
				if(key >> 8 == KEY_SPACE || key >> 8 == KEY_ENTER)
				{
					//SOUND menu forward
					switch(caret_position)
					{
						case 0:
							NewGame();
							GGame.type = GT_FreeForAll;
							RequestState(PreGame::Factory);
							return S_Next;
						case 3: return S_Quit;
					}
				}
			}
			return S_Continue;
	}
	return S_Continue;
}

void Menu::Draw(BITMAP* dest)
{
	switch(curscreen)
	{
		case TitleScreen:
			stretch_blit(gradient, dest, 0, 0, gradient->w, gradient->h, 0, 0, dest->w, dest->h);
			draw_sprite(dest, splash, 50, 50);
			textprintf_centre(dest, font, SCREEN_W / 2, SCREEN_H * 3 / 4, 0,
				"Press a key to begin");
			break;
			
		case MainScreen:
			if(full_redraw)
			{
				stretch_blit(fade_gradient, dest, 0, 0, gradient->w, gradient->h, 0, 0, dest->w, dest->h);
				textprintf_centre(dest, font, SCREEN_W / 2, SCREEN_H * 2 / 3, 0,
					"Multiplayer Game");
				textprintf_centre(dest, font, SCREEN_W / 2, (SCREEN_H * 2 / 3) + (caret_height / 2), 0,
					"1 on 1");
				textprintf_centre(dest, font, SCREEN_W / 2, (SCREEN_H * 2 / 3) + (2 * caret_height / 2), 0,
					"Single player");
				textprintf_centre(dest, font, SCREEN_W / 2, (SCREEN_H * 2 / 3) + (3 * caret_height / 2), 0,
					"Quit");
				full_redraw = false;
			}
			stretch_blit(fade_gradient, dest, SCREEN_W / 2 - 110, 0, caret_width, 1, SCREEN_W / 2 - 110, (SCREEN_H * 2 / 3) + (caret_position * caret_height / 2) - (caret_height / 2), caret_width, caret_height);
			DrawCaret(SCREEN_W / 2 - 110, (SCREEN_H * 2 / 3) + (caret_position * caret_height / 2) - (caret_height / 2));
			break;
	}
}

// The end
